package edu.chl.hajo.shop.core;

import java.io.Serializable;
import java.util.Date;
import java.util.List;
import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.EnumType;
import javax.persistence.Enumerated;
import javax.persistence.Id;
import javax.persistence.JoinColumn;
import javax.persistence.ManyToOne;
import javax.persistence.OneToMany;
import javax.persistence.Temporal;
import lombok.EqualsAndHashCode;
import lombok.Getter;
import lombok.NoArgsConstructor;
import lombok.Setter;

/**
 * A PurchaseOrder (because Order is reserved word in SQL)
 *
 * @author hajo
 */
@NoArgsConstructor
@EqualsAndHashCode(of = {"orderNumber"})
@Entity
public class PurchaseOrder implements Serializable  {

    // State of order
    public enum State {

        ACCEPTED,
        CANCELLED,
        INVOICED,
        SHIPPED,
    }
    @Id
    @Getter 
    @Column( nullable = false )
    private String orderNumber;   
    @Getter
    @Temporal(javax.persistence.TemporalType.DATE)
    @Column(name = "ORDER_DATE", nullable = false)
    private Date date = new Date();
    @Getter
    @OneToMany(cascade = CascadeType.ALL)
    @JoinColumn(name = "PURCHASE_ORDER_ID")
    private List<OrderItem> items;  
    @Getter
    @ManyToOne
    @JoinColumn(name = "CUSTOMER_ID", nullable = false)
    private Customer customer;
    @Getter
    @Setter
    @Column(name = "ORDER_STATE", nullable = false)
    @Enumerated(EnumType.STRING)
    private State state;

    public PurchaseOrder(String orderNumber, List<OrderItem> items, 
            Customer customer, State state) {
        this.orderNumber = orderNumber;
        this.items = items;
        this.customer = customer;
        this.state = state;
    }

 
}
