/*
      The view for a single Person (row)
      (created and appended to #personList tbody")
      Children of appView

*/

define([
    'jquery', 'underscore', 'backbone', 'mustache', 'text!templates/person.html'
], function($, _, Backbone, Mustache, personTemplate) {
    'use strict';

    var PersonView = Backbone.View.extend({
        /* Backbone's view events are scope to the view's el. If el shared by children
          view's ALL children will get the event (i.e. destroy!!!).
          To avoid use tagName, will generate unique parent for each subview
        */
        tagName: 'tr', // Will wrap template inside <tr>

        // Template for this view
        template: Mustache.parse(personTemplate),

        // Connect events and listeners (incomming from
        // GUI)
        events: {
            'click .delete': 'delete',
            'dblclick .edit': 'edit',
            'keypress .edit': 'update'
        },

        // Constructor
        initialize: function(person) {
            this.person = person;
            this.listenTo(this.person, 'destroy', this.remove);
            this.listenTo(this.person, 'all', this.spy);
            this.$el.on("click", this.spy);
        },

        // How to render this view
        render: function() {
            var m = Mustache.render(personTemplate, {"person": this.person});
            // Append all <td>'s' to the <tr> (created by tagName)
            this.$el.append(m);
        },
        // GUI listener
        delete: function(e) {
            this.person.destroy({url: this.person.url()});
        },
        // GUI listener
        edit: function() {
            var input = this.$('.edit');
            input.removeAttr("disabled");
            input.focus();
        },
        // GUI listener
        update: function(e) {
            var input = this.$('.edit');
            if (e.which !== 13 || !input.val().trim()) {
                return;
            }
            var input = this.$('.edit');
            input.attr("disabled", "disabled");
            this.person.set({name: input.val()});
        },
        // Utility
        spy: function(e) {
            console.log("personView spy. " + e + " id = " + this.id + " el = " + this.tagName);
        }
    });

    return PersonView;
});
