/*

	A single model object

*/
// Define dependencies
define([
    'underscore', 'backbone'
], function(_, Backbone) { // Do injection
    'use strict';

    // Object for generating serial numbers (optional, database will handle)
    var counter = {
        value: 100,
        getNext: function() {
            var i = this.value;
            this.value++;
            return i;
        }
    }
    /*
          Many functions defined by Backbone (i.e. there are hidden functions)
          - Person.destroy()
          - Person.set( ...)
          - etc. see Backbone docs.
        */
    var Person = Backbone.Model.extend({
        defaults: {
            name: "unknown"
        },

        // Contructor
        initialize: function(id, name) {
            // Will set attributes, see
            // model.attributes
            this.set("id", !id
                ? counter.getNext()
                : id);
            // MUST use set for attributes
            this.set("name", name);

        }

    });

    return Person;

})
