/*
    The single (main) view for index.html
    Parent to personViews (but no explicit references between parent and children)
*/
define([
    'jquery', 'underscore', 'backbone', 'views/personView'
], function($, _, Backbone, PersonView) {
    'use strict';

    var AppView = Backbone.View.extend({
        // The html element in index.html to attach to (this is a single view)
        el: "#personApp",

        // No template used for this (everything
        // in index.html)

        // Connect events and listeners (incoming from GUI)
        events: {
            'keypress #newPerson': 'createOnEnter'
        },

        // Constructor
        initialize: function(personList) {
            this.personList = personList; // The collection reference

            // Cache some jQuery objects (efficency)
            this.$newPerson = this.$('#newPerson');
            this.$tbody = this.$('#personTable tbody');

            // Set this as observer of "add" events
            // in collection Call this.renderPerson
            // at event
            this.listenTo(this.personList, 'add', this.renderPerson);
            this.listenTo(this.personList, 'all', this.spy);
            // This is just to get the hard coded persons
            this.render();
        },

        // Render list of persons
        render: function() {
            var self = this;
            this.personList.each(function(person) {
                self.renderPerson(person);
            });
        },

        // GUI listener
        createOnEnter: function(e) {
            if (e.which !== 13 || !this.$newPerson.val().trim()) {
                return;
            }

            var name = this.$newPerson.val().trim();
            // Create person with no id and name
            this.personList.create(null, name);
            // Clear GUI
            this.$newPerson.val('');
        },

        // Render single person
        renderPerson: function(person) {
            // Object will be hooked up to event system in initialize.
            // So will live on.
            // This reference will disappear
            var view = new PersonView(person);
            view.render();
            // Append a row  (=view.el) to "#personTable body"
            this.$tbody.append(view.el);
        },
        // Utility
        spy: function(e) {
            console.log("appView spy " + e);
        }
    });

    return AppView;
});
