package edu.chl.hajo.cycle;

import java.util.List;
import javax.faces.component.UIComponent;
import javax.faces.component.UIViewRoot;
import javax.faces.context.FacesContext;
import javax.faces.event.PhaseEvent;
import javax.faces.event.PhaseId;
import javax.faces.event.PhaseListener;
import javax.servlet.http.HttpServletRequest;
import static edu.chl.hajo.cycle.LogUtil.*;

/**
 * Not an attached or managed bean object (can't use
 *
 * @PostContruct).
 *
 * See faces-config.xml See use of FacesContext below
 *
 * @author localhajo
 */
public class MyPhaseListener implements PhaseListener {

    // Dump component tree or not
    private boolean dump = false;

    public MyPhaseListener() {
        isAlive(this, MAGENTA);
    }

    @Override
    public PhaseId getPhaseId() {
        return PhaseId.ANY_PHASE;
    }

    @Override
    public void beforePhase(PhaseEvent event) {
        if (event.getPhaseId() == PhaseId.RESTORE_VIEW) {

            if (event.getFacesContext().isPostback()) {
                info("------- THIS IS A POSTBACK ----------");
                HttpServletRequest req = (HttpServletRequest) event.getFacesContext().
                        getExternalContext().getRequest();
                info("To: " + req.getRequestURL().toString());
                UIViewRoot viewRoot = event.getFacesContext().getViewRoot();
                if (viewRoot != null) {
                    info("View id: " + viewRoot.getViewId());
                }

            } else {
                info("------- THIS IS AN INITIAL REQUEST ----------");
            }

        } else if (event.getPhaseId() == PhaseId.APPLY_REQUEST_VALUES) {
            if (dump) {
                FacesContext fC = event.getFacesContext();
                UIViewRoot root = fC.getViewRoot();
                if (root != null) {
                    info("View id is " + root.getViewId());
                    info("Number of children are : " + root.getChildCount());
                    dumpTree(root, "-");
                    System.out.println();
                }
            }
        }
        LogUtil.info("Before: " + event.getPhaseId(), BLUE);
    }

    @Override
    public void afterPhase(PhaseEvent event) {
        info("After: " + event.getPhaseId(), BLUE);
        //LogUtil.info("View root: " + event.getFacesContext().getViewRoot());
    }

    // Print out components in component tree
    private void dumpTree(UIComponent comp, String indent) {
        if (comp.getChildCount() != 0) {
            List<UIComponent> children = comp.getChildren();
            for (UIComponent c : children) {
                info(indent + " Child " + c.getId()
                        + " family " + c.getFamily(), CYAN);
                dumpTree(c, indent + " > ");
            }
        }
    }
}
