/*
  A customized built-in element is a custom element that extends one of the
  browser's built-in HTML tags. The primary benefit of extending an existing element
  is to gain all of its features (DOM properties, methods, accessibility).
  There's no better way to write a progressive web app than to progressively
  enhance existing HTML elements.
*/

// This is ES6, a class (just syntactic sugar)
class LoginElement extends HTMLElement {

    // An instance of the element is created or upgraded
    constructor() {
        super(); // Always first, establish prototype chain
    }

    // Called every time the element is inserted into the DOM.
    connectedCallback() {
        console.log("ID is " + this.id);

        this.style.cursor = 'pointer';
        this.style.userSelect = 'none';
        this.style.webkitUserSelect = 'none';
        this.style.MozUserSelect = 'none';

        // Order matters!!!
        this.attachShadow({mode: 'open'});
        this.render();
        // Store part of component for later use
        this.email = this.shadowRoot.querySelector('#email');
        this.passwd = this.shadowRoot.querySelector('#passwd');
        this.btn = this.shadowRoot.querySelector('#button');
    }

    // Called every time the element is removed from the DOM
    disconnectedCallback() {
        //this.removeEventListener('click', this.onClick);
        // Will remove event listeners?
        this.shadowRoot.innerHTML = '';
    }

    render() {
        // Also works in Chromw (old spec)
        //this.createShadowRoot().innerHTML...
        //this.attachShadow({mode: 'open'}).innerHTML = `
        this.shadowRoot.innerHTML = `
            <div id='content'>
            <p>Please log in </p>
            <table>
            <tr>
              <td>Email</td>
              <td><input id='email' type='text'/></td>
            </tr>
            <tr>
              <td>Password</td>
              <td><input id='passwd' type='text'/></td>
            </tr>
            <tr><td><input id='button' type='button' value='Login' /></td></tr>
          </table>
          </div>`;
    }

    // Pre: listener must be able to handle email passwd args
    addEventListener(listener) {
        let self = this;
        this.btn.addEventListener('click', function(e) {
            listener(self.email.value, self.passwd.value);
        });
    }
}

/*
  The name of a custom element must contain a dash (-)
  You can't register the same tag more than once
  Custom elements cannot be self-closing
*/
customElements.define('log-in', LoginElement);
